//
//  ISViewController.m
//  ExpandableTextCell
//
//  Created by Simon Liu on 17/6/8.
//  Copyright © 2017年 xzliu. All rights reserved.
//

#import "ISViewController.h"
#import "ISExpandableTextCell.h"
#import <ISBankCard/ISBankCard.h>
#import <ISOpenSDKFoundation/ISOpenSDKFoundation.h>
#import "ISZoomScrollView.h"

#define IS_DEVICE_HEIGHT    [[UIScreen mainScreen] bounds].size.height
#define IS_DEVICE_WIDTH     [[UIScreen mainScreen] bounds].size.width

#define COLOR_WITH_HEX(hexValue) [UIColor colorWithRed:((float)((hexValue & 0xFF0000) >> 16)) / 255.0 green:((float)((hexValue & 0xFF00) >> 8)) / 255.0 blue:((float)(hexValue & 0xFF)) / 255.0 alpha:1.0f]
@implementation ISOCRResultItem

-(id) init
{
    self = [super init];
    if (self) {
        _typeText = nil;
        _resultText = nil;
    }
    return self;
}

@end

@interface ISViewController ()<ISExpandableTableViewDelegate>
{
    CGFloat _cellHeight[1000];
    BOOL _loadImageBOOL;
    
    NSArray *_infoImageArray;
    NSDictionary *_OCRDict;
    NSArray *_OCRArray;
    
}

@property (nonatomic, strong) NSMutableArray *cellData;
@property (nonatomic, strong) NSMutableArray *cellDataKey;
@property (nonatomic, assign) ISCreditCardType creditCardType;

@property (strong, nonatomic) NSDictionary *infoDict;
@property (strong, nonatomic) UIImage *originImage;
@property (strong, nonatomic) UIImage *headImage;
@property (strong, nonatomic) UIImage *typeImage;
@property (strong, nonatomic) ISZoomScrollView *zoomScrollView ;

@end

@implementation ISViewController

- (instancetype)initWithDictionary:(NSDictionary *)info loadImageBool:(BOOL)loadImageBOOL
{
    self = [super init];
    if (self != nil) {
        self.originImage = [info valueForKey:kOpenSDKCardResultTypeImage];
        self.infoDict = info;
        _loadImageBOOL=loadImageBOOL;
        self.typeImage = [info valueForKey:kOpenSDKCardResultTypeOriginImage];
        
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.navigationItem.title = @"银行卡识别结果";
    //字体颜色
    self.navigationController.navigationBar.tintColor = [UIColor grayColor];
    self.navigationController.navigationBar.backgroundColor = [UIColor whiteColor];
    self.tableView.backgroundColor = [UIColor colorWithRed:241.0/255.0 green:241.0/255.0 blue:241.0/255.0 alpha:1];
    self.tableView.separatorColor =[UIColor colorWithRed:241.0/255.0 green:241.0/255.0 blue:241.0/255.0 alpha:1];
    self.tableView.separatorInset = UIEdgeInsetsMake(0, 15, 0, 15);//分割线缩进 两端距离
    
    //获取OCR结果显示
    [self setupDataSource];
    
    UITapGestureRecognizer * tapGesture = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(dismissKeyBoard)];
    tapGesture.cancelsTouchesInView = NO;
    [self.tableView addGestureRecognizer:tapGesture];
    
    UIBarButtonItem *leftBarItem =  [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"IS_Market_Back"] style:UIBarButtonItemStyleDone target:self action:@selector(clickLeftBarButtonItem)];
    self.navigationItem.leftBarButtonItem = leftBarItem;
    UIView *header = [[UIView alloc] initWithFrame:CGRectMake(0, 0, IS_DEVICE_WIDTH, IS_DEVICE_HEIGHT*2/7)];
    [self setUpZoomScrollView:header];
    self.tableView.tableHeaderView = header;
    
}
- (void)setupDataSource
{
    self.cellData =[[NSMutableArray alloc] init];
    self.cellDataKey =[[NSMutableArray alloc] init];
    {
        UIImage *cardNumImage = self.infoDict[kBankNumberImage];
        UIImage *typeImage = self.infoDict[kOpenSDKCardResultTypeImage];
        UIImage *originImage = self.infoDict[kOpenSDKCardResultTypeOriginImage];
        _infoImageArray = @[typeImage,originImage,cardNumImage];

    }
    
    self.navigationItem.title = NSLocalizedString(@"BankCard Recognition Result", @"BankCard Recognition Result");
    {
        ISCreditCardType creditType = [[self.infoDict valueForKey:kCreditCardType] integerValue];//信用卡类型
        self.creditCardType = creditType;
        NSString *creditCardTypeStr = [self getCreditCardTypeString:creditType];
        if (creditCardTypeStr) {
            [self.cellData addObject:creditCardTypeStr];
        }
    }
    
    {
        NSString *cardNumber = [self.infoDict valueForKey:kCardNumber];//卡号
        if (cardNumber) {
            ISOCRResultItem *item = [[ISOCRResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Card number:", @"Card number:");
            item.resultText = cardNumber;
            [self.cellData addObject:item];
        }
    }
    
    {
        NSString *expiryDate = [self.infoDict valueForKey:kExpiryDate];//过期时间
        if (expiryDate) {
            ISOCRResultItem *item = [[ISOCRResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Valid Thru:", @"Valid Thru");
            item.resultText = expiryDate;
            [self.cellData addObject:item];
        }
    }
    
    {
        NSString *bankName = [self.infoDict valueForKey:kBankName];//银⾏名
        NSString *bankID = [self.infoDict valueForKey:kBankID];
        if (bankID == nil) {
            bankID = NSLocalizedString(@"Unknown Bank", @"Unknown Bank");
        }
        if (bankName) {
            ISOCRResultItem *item = [[ISOCRResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Issued bank:", @"Bank");
            NSString *bankNameString = [NSString stringWithFormat:@"%@(%@)", bankName, bankID];
            item.resultText = bankNameString;
            [self.cellData addObject:item];
        }
    }
    
    {
        ISBankCardType bankCardType = [[self.infoDict valueForKey:kBankCardType] integerValue];//银⾏卡类型
        NSString *bankCardTypeStr = [self getBankCardTypeString:bankCardType];
        if (bankCardTypeStr) {
            ISOCRResultItem *item = [[ISOCRResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Card type:", @"Card type:");
            item.resultText = bankCardTypeStr;
            [self.cellData addObject:item];
        }
    }
    
    {
        NSString *holderName = [self.infoDict valueForKey:kHolderName];//持卡人姓名
        
        if (holderName) {
            ISOCRResultItem *item = [[ISOCRResultItem alloc] init];
            item.typeText =NSLocalizedString(@"Holder Name:", @"Holder Name:");
            NSString *holderNameString = [NSString stringWithFormat:@"%@", holderName];
            item.resultText = holderNameString;
            [self.cellData addObject:item];
        }
    }
    
    {
        NSInteger rotate = [[self.infoDict valueForKey:kOpenSDKCardResultTypeCardRotate] integerValue];//图片旋转角
        NSLog(@"Image rotate is %ld", (long)rotate);
    }
    
}

- (NSString*) getBankCardTypeString:(ISBankCardType)type
{
    NSString *cardString = nil;
    switch (type) {
        case ISBankCardTypeCreditCard:
            cardString = NSLocalizedString(@"CreditCard", @"CreditCard");
            break;
        case ISBankCardTypeDebitCard:
            cardString = NSLocalizedString(@"DebitCard", @"DebitCard");
            break;
        case ISBankCardTypeQuasiCreditCard:
            cardString = NSLocalizedString(@"QuasiCreditCard", @"QuasiCreditCard");
            break;
        default:
            break;
    }
    return cardString;
}

- (NSString*) getCreditCardTypeString:(ISCreditCardType)type
{
    NSString *cardString = nil;
    switch (type) {
        case ISCreditCardTypeVISA:
            cardString = NSLocalizedString(@"VISA", @"VISA");
            break;
        case ISCreditCardTypeMASTER:
            cardString = NSLocalizedString(@"MASTER", @"MASTER");
            break;
        case ISCreditCardTypeMAESTRO:
            cardString = NSLocalizedString(@"MAESTRO", @"MAESTRO");
            break;
        case ISCreditCardTypeAMEX:
            cardString = NSLocalizedString(@"AMEX", @"AMEX");
            break;
        case ISCreditCardTypeDINERS:
            cardString = NSLocalizedString(@"DINERS", @"DINERS");
            break;
        case ISCreditCardTypeDISCOVER:
            cardString = NSLocalizedString(@"DISCOVER", @"DISCOVER");
            break;
        case ISCreditCardTypeJCB:
            cardString = NSLocalizedString(@"JCB", @"JCB");
            break;
        case ISCreditCardTypeUNIONPAY:
            cardString = NSLocalizedString(@"UNIONPAY", @"UNIONPAY");
            break;
        case ISCreditCardTypeNone:
            cardString = NSLocalizedString(@"", @"");
            break;
        default:
            break;
    }
    return cardString;
}

- (UIImage*) getCreditCardTypeIcon:(ISCreditCardType)type
{
    UIImage *image = nil;
    switch (type) {
        case ISCreditCardTypeVISA:
            image = [UIImage imageNamed:@"VISA"];
            break;
        case ISCreditCardTypeMASTER:
            image = [UIImage imageNamed:@"mastercard"];
            break;
        case ISCreditCardTypeAMEX:
            image = [UIImage imageNamed:@"AMERICAN"];
            break;
        case ISCreditCardTypeJCB:
            image = [UIImage imageNamed:@"JCB"];
            break;
        case ISCreditCardTypeUNIONPAY:
            image = [UIImage imageNamed:@"Unionpay"];
            break;
        case ISCreditCardTypeNone:
            image = [UIImage imageNamed:@""];
            break;
        default:
            break;
    }
    return image;
}

#pragma mark - Table View Data Source
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.cellData.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSString *CellIdentifier = [NSString stringWithFormat:@"Cell%ld%ld", (long)[indexPath section], (long)[indexPath row]];//以indexPath来唯一确定cell
    ISExpandableTextCell *cell = [tableView expandableTextCellWithId:CellIdentifier];
    
    if (indexPath.row==0)
    {
        UIImageView *creditTypeImageView = [[UIImageView alloc] init];
        creditTypeImageView.frame = CGRectMake(20, 11, 36, 22);
        creditTypeImageView.image = [self getCreditCardTypeIcon:self.creditCardType];
        [cell.contentView addSubview:creditTypeImageView];
        cell.textView.text = self.cellData[indexPath.row];
        
    }else
    {
        if (indexPath.row < self.cellData.count) {
            NSObject *data = self.cellData[indexPath.row];
            if ([data isKindOfClass:[NSString class]]) {
                cell.leftTextView.text = (NSString *)data;
            } else if ([data isKindOfClass:[ISOCRResultItem class]]) {
                ISOCRResultItem *item = (ISOCRResultItem*)data;
                cell.leftTextView.text = item.typeText;
                cell.textView.text = item.resultText;
            }
        }
    }
    
    
    return cell;
}


#pragma mark - Table View Delegate
- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    if (indexPath.row == 0)
    {
        return 44.0f;
    }else{
        ISOCRResultItem *resultItem = self.cellData[indexPath.row];
        NSString* text = resultItem.resultText;
        CGSize constraint = CGSizeMake(self.view.frame.size.width-135, MAXFLOAT);
        NSAttributedString* attributedText = [[NSAttributedString alloc]initWithString:text attributes:@{NSFontAttributeName:[UIFont systemFontOfSize:14]}];
        CGRect rect = [attributedText boundingRectWithSize:constraint options:NSStringDrawingUsesLineFragmentOrigin context:nil];
        CGSize size = rect.size;
        CGFloat height = MAX(size.height+26.5, _cellHeight[indexPath.row]);
        return height;
    }
    
}

- (void)tableView:(UITableView *)tableView updatedHeight:(CGFloat)height atIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.row!=0)
    {
        _cellHeight[indexPath.row] = height;
    }
}

- (void)tableView:(UITableView *)tableView updatedText:(NSString *)text atIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.section!=0)
    {
        [_cellData replaceObjectAtIndex:indexPath.row withObject:text];
    }
}

- (void)setUpZoomScrollView:(UIView *)addView
{
    self.zoomScrollView = [ISZoomScrollView roundScrollViewWithFrame:addView.frame imageArr:_infoImageArray timerWithTimeInterval:5 imageClick:^(NSInteger imageIndex) {
    }];
    [addView addSubview:self.zoomScrollView];
}
- (void)clickLeftBarButtonItem
{
    [self.zoomScrollView removeFromSuperview];
    [self dismissViewControllerAnimated:YES completion:nil];
    
}

-(void)dismissKeyBoard{
    [self.view endEditing:YES];
}




@end

