//
//  ISZoomScrollView.m
//  ISIDReaderCameraPreviewDemo
//
//  Created by Simon Liu on 17/3/6.
//  Copyright © 2017年 Felix Liu. All rights reserved.
//

#import "ISZoomScrollView.h"

#define SCREEN_WIDTH [UIScreen mainScreen].bounds.size.width
#define SCREEN_HEIGHT [UIScreen mainScreen].bounds.size.height

@interface ISZoomScrollView ()<UIScrollViewDelegate>
{
    CGFloat _offset;
    int _centerCurIndex;
}
@property(nonatomic,assign)NSInteger imgCount;//图片总个数
@property(nonatomic,copy)NSArray *imageArr;//图片的数组
@property(nonatomic,assign)NSTimeInterval timeInterVal;//图片滑动时间
@property(nonatomic,copy)imageClick imgClick;//图片点击block
@property(nonatomic,retain)UIPageControl *pageControl;//小圆点

@property(nonatomic,retain)UIScrollView *imageScrollView;

@end


@implementation ISZoomScrollView

#pragma mark---实现类方法
+(instancetype)roundScrollViewWithFrame:(CGRect)frame imageArr:(NSArray *)imageArr timerWithTimeInterval:(NSTimeInterval)timeInterVal imageClick:(imageClick)imgClick;
{
    ISZoomScrollView *roundScrollView=[[ISZoomScrollView alloc]initWithFrame:frame];
    roundScrollView.imageArr=imageArr;
    roundScrollView.timeInterVal=timeInterVal;
    roundScrollView.imgClick=imgClick;
    return roundScrollView;
}
#pragma mark---属性重新赋值
-(void)setImageArr:(NSArray *)imageArr
{
    _imageArr=imageArr;
    _imgCount=_imageArr.count;
    [self addScrollView];
//    [self addImageView];
    [self addPageControl];
}

- (void)addScrollView
{
    _centerCurIndex=0;

    self.imageScrollView=[[UIScrollView alloc]initWithFrame:CGRectMake(0, 0, self.bounds.size.width, self.bounds.size.height)];
    self.imageScrollView.pagingEnabled=YES;
    self.imageScrollView.scrollEnabled = YES;
    self.imageScrollView.delegate=self;
    self.imageScrollView.showsVerticalScrollIndicator=NO;
    self.imageScrollView.showsHorizontalScrollIndicator=NO;
    self.imageScrollView.contentSize=CGSizeMake(_imgCount*SCREEN_WIDTH, self.bounds.size.height);
    self.imageScrollView.backgroundColor = [UIColor whiteColor];
    [self addSubview:self.imageScrollView];
    
    
    for (int i = 0; i<_imageArr.count; i++){

        UIScrollView *parentScrollView = [[UIScrollView alloc] initWithFrame:CGRectMake(self.bounds.size.width*i, 0, self.bounds.size.width, self.bounds.size.height)];
        parentScrollView.backgroundColor = [UIColor clearColor];
        parentScrollView.showsHorizontalScrollIndicator = NO;
        parentScrollView.showsVerticalScrollIndicator = NO;
        parentScrollView.delegate = self;
        parentScrollView.minimumZoomScale = 1.0;
        parentScrollView.maximumZoomScale = 3.0;
        parentScrollView.tag = i+1;
        [parentScrollView setZoomScale:1.0];
        
        UIImageView *imageview = [[UIImageView alloc] init];
        imageview.image = _imageArr[i];
        imageview.frame = CGRectMake(0, 0, self.bounds.size.width, self.bounds.size.height);
        [imageview setContentMode:UIViewContentModeScaleAspectFit];
        imageview.userInteractionEnabled = YES;
        imageview.tag = i+100;
        [parentScrollView addSubview:imageview];
        [self.imageScrollView addSubview:parentScrollView];
        

    }
    
}

#pragma mark - ScrollView delegate
-(UIView *)viewForZoomingInScrollView:(UIScrollView *)scrollView{
    
    for (UIView *v in scrollView.subviews){
        return v;
    }
    return nil;
}

-(void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView{
    

    if (scrollView == self.imageScrollView){
        CGFloat x = scrollView.contentOffset.x;
        if (x==_offset){

        }
        else {
            _pageControl.currentPage = _centerCurIndex;
            _offset = x;
            for (UIScrollView *s in scrollView.subviews){
                if ([s isKindOfClass:[UIScrollView class]]){
                    [s setZoomScale:1.0];
                    UIImageView *image = [[s subviews] objectAtIndex:0];
                    image.frame = CGRectMake(0, 0, self.bounds.size.width, self.bounds.size.height);
                }
            }
        }
    }
}

- (void)scrollViewDidEndZooming:(UIScrollView *)scrollView withView:(UIView *)view atScale:(CGFloat)scale {
    
}

-(void)scrollViewDidZoom:(UIScrollView *)scrollView{
    UIView *v = [scrollView.subviews objectAtIndex:0];
    if ([v isKindOfClass:[UIImageView class]]){
        if (scrollView.zoomScale<1.0){
            //         v.center = CGPointMake(scrollView.frame.size.width/2.0, scrollView.frame.size.height/2.0);
        }
    }
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView
{
    int page = scrollView.contentOffset.x / scrollView.frame.size.width;
    _centerCurIndex=page;
}

#pragma mark -
-(void)handleDoubleTap:(UIGestureRecognizer *)gesture{
    
    float newScale = [(UIScrollView*)gesture.view.superview zoomScale] * 1.5;
    CGRect zoomRect = [self zoomRectForScale:newScale  inView:(UIScrollView*)gesture.view.superview withCenter:[gesture locationInView:gesture.view]];
    UIView *view = gesture.view.superview;
    if ([view isKindOfClass:[UIScrollView class]]){
        UIScrollView *s = (UIScrollView *)view;
        [s zoomToRect:zoomRect animated:YES];
    }
}

#pragma mark - Utility methods

-(CGRect)zoomRectForScale:(float)scale inView:(UIScrollView*)scrollView withCenter:(CGPoint)center {
    
    CGRect zoomRect;
    
    zoomRect.size.height = [scrollView frame].size.height / scale;
    zoomRect.size.width  = [scrollView frame].size.width  / scale;
    
    zoomRect.origin.x    = center.x - (zoomRect.size.width  / 2.0);
    zoomRect.origin.y    = center.y - (zoomRect.size.height / 2.0);
    
    return zoomRect;
}

-(CGRect)resizeImageSize:(CGRect)rect{
    NSLog(@"x:%f y:%f width:%f height:%f ", rect.origin.x, rect.origin.y, rect.size.width, rect.size.height);
    CGRect newRect;
    
    CGSize newSize;
    CGPoint newOri;
    
    CGSize oldSize = rect.size;
    if (oldSize.width>=self.bounds.size.width || oldSize.height>=self.bounds.size.height){
        float scale = (oldSize.width/self.bounds.size.width>oldSize.height/460.0?oldSize.width/self.bounds.size.width:oldSize.height/self.bounds.size.height);
        newSize.width = oldSize.width/scale;
        newSize.height = oldSize.height/scale;
    }
    else {
        newSize = oldSize;
    }
    newOri.x = (self.bounds.size.width-newSize.width)/2.0;
    newOri.y = (self.bounds.size.height-newSize.height)/2.0;
    
    newRect.size = newSize;
    newRect.origin = newOri;
    
    return newRect;
}


- (void)addPageControl
{
    _pageControl=[[UIPageControl alloc]init];
    //注意此方法可以根据页数返回UIPageControl合适的大小
    CGSize size= [_pageControl sizeForNumberOfPages:_imgCount];
    _pageControl.bounds=CGRectMake(0, 0, size.width, size.height);
    _pageControl.center=CGPointMake(SCREEN_WIDTH/2, self.bounds.size.height-size.height/2);
    _pageControl.pageIndicatorTintColor=[UIColor colorWithWhite:0.846 alpha:1.000];
    _pageControl.currentPageIndicatorTintColor=[UIColor colorWithWhite:0.600 alpha:1.000];
    _pageControl.numberOfPages=_imgCount;
    _pageControl.currentPage=0;

    [self addSubview:_pageControl];
}
//#pragma mark---小圆点控制器位置
//-(void)setPageControlAlignment:(NSPageControlAlignment)pageControlAlignment
//{
//    CGSize size= [_pageControl sizeForNumberOfPages:_imgCount];
//    
//    if (pageControlAlignment==NSPageControlAlignmentCenter) {
//        _pageControl.center=CGPointMake(SCREEN_WIDTH/2, self.bounds.size.height-size.height/2);
//    }else if(pageControlAlignment==NSPageControlAlignmentRight){
//        _pageControl.center=CGPointMake(self.bounds.size.width-size.width/2-20, self.bounds.size.height-size.height/2);
//    }
//}
//#pragma mark---当前小圆点颜色
//-(void)setCurPageControlColor:(UIColor *)curPageControlColor
//{
//    _pageControl.currentPageIndicatorTintColor=curPageControlColor;
//}
//#pragma mark---其余小圆点颜色
//-(void)setOtherPageControlColor:(UIColor *)otherPageControlColor
//{
//    _pageControl.pageIndicatorTintColor=otherPageControlColor;
//}

@end
